function [totalEval InferParam HyperParam] = do_inference(configFile)
% This is the main file to run inference for GTCM
% Usage: 
%   after running initpaths run with:
%   [subjectResults totalEval InferParam HyperParam] = do_inference('config')
% 
% Author: Richard Socher
eval(configFile);

%%% read in LDA and Kahana data
if ~exist('LDA','var')
    % useCorpus is set in the config file
    LDA = load_LDA(useCorpus);
end

if ~exist('Data', 'var')
    Data = load(PerDataSetFolder);
    Data = Data.data;
    nounpool = lower(Data.nounpool);
    
    subjectIDs = {};
    
    LDA.oriVocabEnd =  length(LDA.vocab);
    LDA.vocab = [LDA.vocab ; setdiff(nounpool,LDA.vocab)'];
    
end

missingWordsInStudyList = [];
missingWordsInRecallList = [];
maxMiss = [];
numOfTrials = [];
numSubjects = 0;
allParticles = cell(InferParam.numOfParticles,1);


if InferParam.oneParamSetForAll
    disp('Sampling particle parameter set for all subjects')
    for c=1:InferParam.numOfParticles
        allParticles{c}.subjectParameters = sampleSubjectParameters(HyperParam);
    end
end

for subjectID = Data.subjlist(1:InferParam.numOfSubjects)
    numOfTrialsForThisSubject = 0;
    
    if ~InferParam.oneParamSetForAll && InferParam.subjectHier
        disp('Resampling subjects'' particles parameters')
        for c=1:InferParam.numOfParticles
            allParticles{c}.subjectParameters = sampleSubjectParameters(HyperParam);
        end
    end
    
    for trialID = 1:InferParam.numOfTrials
        list_pre = Data.wordindex(ismember(Data.subject, subjectID) & ismember(Data.type, 'STUDY_WORD') & ...
            Data.block == trialID+1 & Data.listType == InferParam.listType);
        list_pre_recall = Data.wordindex(ismember(Data.subject, subjectID) & ismember(Data.type, 'FR_RESP') & ...
            Data.block == trialID+1 & Data.listType == InferParam.listType);
        
        if ~isempty(list_pre_recall)
            list = getLDAWordNumbers(nounpool(list_pre),LDA.vocab);
            listR = getLDAWordNumbers(nounpool(list_pre_recall),LDA.vocab);
            
            missingWordsInStudyList(end+1) = sum(list>LDA.oriVocabEnd);
            missingWordsInRecallList(end+1) = sum(listR> LDA.oriVocabEnd);
            maxMiss(end+1) = max(missingWordsInStudyList(end),missingWordsInRecallList(end));
            
            
            %----------------------
            % check list properties
            if InferParam.onlyCleanRecallLists && length(list) ~= length(list_pre)
                disp('Words are missing, skipping this list!')
                continue;
            end
            
            % check if words occur multiple times during recall and ignore those lists
            if InferParam.skipListsWithMultipleEqualWords && any(histc(listR,1:length(LDA.vocab)) > 1)
                disp('Multiple equal recalled words, skipping this list!')
                continue;
            end
            
            if maxMiss(end) > InferParam.maxMissingWords
                disp(['Subject: ' subjectID{1} ' trial: ' num2str(trialID) ...
                    ', LDA-missing words: ' num2str(maxMiss(end)) ' (> ' num2str(InferParam.maxMissingWords ) ', skipping this list)'])
                continue;
            end            
            %----------------------
            
            
            if ~ismember(subjectID,subjectIDs)
                subjectIDs = [subjectIDs subjectID];
                numSubjects= numSubjects+1;
            end
            numOfTrialsForThisSubject = numOfTrialsForThisSubject+1;
            
            %----------------------
            % run particles through the study simulation
            %----------------------
            for c=1:InferParam.numOfParticles
                if InferParam.subjectHier
                    % already sampled for this subject or coming from last trial
                elseif InferParam.oneParamSetForAll
                    % already sampled for all subjects
                else
                    allParticles{c}.subjectParameters = sampleSubjectParameters(HyperParam);
                end
                prm.eta1 = allParticles{c}.subjectParameters.eta_s1;
                prm.sigma = allParticles{c}.subjectParameters.sigma;
                [studyWordList theta_s] = sim_study(prm,LDA,list);
                allParticles{c}.theta = theta_s;
                if isfield(allParticles{c},'phi')
                    allParticles{c} = rmfield( allParticles{c}, {'phi','weight','stheta'});
                end
            end
            
            
            %----------------------
            % Run inference over the recall list
            %----------------------
            [particleAvg particleEval allParticles] = infer_recall(configFile, LDA, studyWordList, listR, allParticles);
            
            %----------------------
            % For evaluation only
            %----------------------            
            subjectResults{numSubjects}.trials{numOfTrialsForThisSubject}.particleAvg = particleAvg;
            subjectResults{numSubjects}.trials{numOfTrialsForThisSubject}.particleEval = particleEval;
            subjectResults{numSubjects}.trials{numOfTrialsForThisSubject}.list = list;
            subjectResults{numSubjects}.trials{numOfTrialsForThisSubject}.listR = listR;
            subjectResults{numSubjects}.trials{numOfTrialsForThisSubject}.list_pre = list_pre;
            subjectResults{numSubjects}.trials{numOfTrialsForThisSubject}.list_pre_recall= list_pre_recall;
            
            disp(['Processed subject:' num2str(numSubjects) ', trial:' num2str(numOfTrialsForThisSubject)])
            
        end
    end
    
    %-----------------
    % Subject evaluation
    %-----------------
    if numOfTrialsForThisSubject>0
        numOfTrials(end+1) = numOfTrialsForThisSubject;
        %TODO:       disp('averaging subject specific parameters Omega...')
        totalSubjAPE(numSubjects) = 0;
        totalSubjAPE_chance_onlyStudyWords(numSubjects) = 0;
        totalSubjAPE_chance_allLDAwords(numSubjects) = 0;
        totalSubjESS(numSubjects) = 0;
        listOfAPEs = [];
        
        for t = 1:length(subjectResults{numSubjects}.trials)
            totalSubjAPE(numSubjects) = totalSubjAPE(numSubjects) + sum(subjectResults{numSubjects}.trials{t}.particleEval.APE);
            listOfAPEs = [listOfAPEs sum(subjectResults{numSubjects}.trials{t}.particleEval.APE)];
            totalSubjESS(numSubjects) = totalSubjESS(numSubjects) + sum(subjectResults{numSubjects}.trials{t}.particleEval.ESS);
            totalSubjAPE_chance_onlyStudyWords(numSubjects) = totalSubjAPE_chance_onlyStudyWords(numSubjects) + ...
                sum(subjectResults{numSubjects}.trials{t}.particleEval.APE_chance_onlyStudyWords);
            totalSubjAPE_chance_allLDAwords(numSubjects) = totalSubjAPE_chance_allLDAwords(numSubjects) + ...
                sum(subjectResults{numSubjects}.trials{t}.particleEval.APE_chance_allLDAwords);
        end
        disp(['subject ' num2str(numSubjects) ' had ' num2str(numOfTrialsForThisSubject) ' trial(s), sum(APE): ' ...
            num2str(sum(totalSubjAPE)) ', length(list_recall)= ' num2str(length(subjectResults{numSubjects}.trials{t}.particleEval.APE)) ' (' num2str(listOfAPEs, '%6.2f ') ')'])
        
        disp(['saving subjectResults and particleAvg to file: ' ])
        save(fullfile('output',['subj' num2str(numSubjects) '.mat']),'particleAvg','subjectResults' ) %allParticles
        clear particleAvg subjectResults;
        
    end
    
end


%-----------------
% Global evaluation
%-----------------

totalEval.totalSubjAPE = totalSubjAPE;
totalEval.totalSubjESS = totalSubjESS;
totalEval.totalAPE = sum(totalSubjAPE);
totalEval.totalESS = sum(totalSubjESS);
totalEval.totalSubjAPE_chance_onlyStudyWords = sum(totalSubjAPE_chance_onlyStudyWords);
totalEval.totalSubjAPE_chance_allLDAwords = sum(totalSubjAPE_chance_allLDAwords);
totalEval.maxMiss = maxMiss;

disp(['number of fitted & evaluated trials: ' num2str(sum(numOfTrials)) ])
totalEval.missingWordsInStudyList = missingWordsInStudyList;
totalEval.missingWordsInRecallList = missingWordsInRecallList;

save('evaluation.mat','totalEval');

